#' Grafic user interface illustrating the central limit theorem
#'
#' Grafic user interface for the central limit theorem. Illustrates how sums
#' and means of several standard distributions converge toward Gaussian
#' densities.
#' 
#' @return Nothing
#' @import shiny
#' @export
#' @examples
#' ## Only run these examples in interactive R sessions
#' if( interactive() ){
#'   cgsgui()
#' }
cgsgui <- function(){
    ##general information about exiting the app
    print("Use escape or ctrl+c to exit")

#######################################
    ## Internatl helper functions - GUIs ##
#######################################
    ##Define variable for choices, to fix UNI-code of sigma
    choices <- c("exp", "unif", "norm", "binom", "pois")
    names(choices) <- c("Exp(1/\u03bc)", "Re(a, b)", "N(\u03bc, \u03c3)",
                        "Bin(n, p)", "Po(\u03bc)")


    ##the server for cgsgui
    cgsgui.server <- function(input, output, session){
        ##First a reactive component generating plots
        currentData <- reactive({
            ##require that some parameters are OK
            req(input$N > 0, input$plot)
            ##create return structure
            ret <- list(discrete=FALSE)
            if(input$dens == "exp"){
                ##require that parameters are OK
                req(input$mu_exp > 0)
                ##First define random generator functions
                dfun <- function(x){ dexp(x, 1/input$mu_exp) }
                rgen <- function(n){ rexp(n, 1/input$mu_exp) }
                ##compute mean and standard deviation
                ret$mu <- input$mu_exp
                ret$sd <- input$mu_exp
                ##define x-limits
                ret$xlim <- c(0, qexp(0.999, 1/input$mu_exp))
            }else if(input$dens == "unif"){
                ##require that parameters are OK
                req(input$a_unif < input$b_unif)
                ##First define random generator functions
                dfun <- function(x){ dunif(x, input$a_unif, input$b_unif) }
                rgen <- function(n){ runif(n, input$a_unif, input$b_unif) }
                ##compute mean and standard deviation
                ret$mu <- (input$b_unif+input$a_unif)/2
                ret$sd <- (input$b_unif-input$a_unif) / sqrt(12)
                ##define x-limits
                ret$xlim <- c(input$a_unif-ret$sd, input$b_unif+ret$sd)
            }else if(input$dens == "norm"){
                ##require that parameters are OK
                req(input$mu_norm, input$sigma_norm > 0)
                ##First define random generator functions
                dfun <- function(x){ dnorm(x, input$mu_norm, input$sigma_norm) }
                rgen <- function(n){ rnorm(n, input$mu_norm, input$sigma_norm) }
                ##compute mean and standard deviation
                ret$mu <- input$mu_norm
                ret$sd <- input$sigma_norm
                ##define x-limits
                ret$xlim <- qnorm(c(0.001,0.999), input$mu_norm, input$sigma_norm)
            }else if(input$dens == "binom"){
                ##require that parameters are OK
                req(input$n_binom > 0, input$p_binom > 0, input$p_binom < 1)
                ##First define random generator functions
                dfun <- function(x){ dbinom(x, input$n_binom, input$p_binom) }
                rgen <- function(n){ rbinom(n, input$n_binom, input$p_binom) }
                ##compute mean and standard deviation
                ret$mu <- input$n_binom*input$p_binom
                ret$sd <- sqrt(ret$mu*(1-input$p_binom))
                ##define x-limits
                ret$xlim <- c(0, input$n_binom+1)
                ##set flags
                ret$discrete <- TRUE
            }else if(input$dens == "pois"){
                ##require that parameters are OK
                req(input$mu_pois > 0)
                ##First define random generator functions
                dfun <- function(x){ dpois(x, input$mu_pois) }
                rgen <- function(n){ rpois(n, input$mu_pois) }
                ##compute mean and standard deviation
                ret$mu <- input$mu_pois
                ret$sd <- sqrt(input$mu_pois)
                ##define x-limits
                ret$xlim <- c(0, qpois(0.999, input$mu_pois))
                ##set flags
                ret$discrete <- TRUE
            }
            
            ##compute density function
            if( ret$discrete ){
                ret$x <- seq(floor(ret$xlim[1]), ceiling(ret$xlim[2]), by=1)
            }else{
                ret$x <- seq(ret$xlim[1], ret$xlim[2], length.out=1e3)
            }
            ret$y <- dfun(ret$x)
            ##simulate data
            ret$xSim <- matrix(rgen(1e3*ceiling(input$N)), ncol=1e3)
            ret$xSim.1 <- ret$xSim[,1]
            if( input$isSum ){
                ret$xSim <- colSums(ret$xSim)
            }else{
                ret$xSim <- colMeans(ret$xSim)
            }
            ##compute Gaussian approximation
            if( input$isSum ){
                ret$mu <- ret$mu * ceiling(input$N)
                ret$sd <- ret$sd * sqrt(ceiling(input$N))
            }else{
                ret$sd <- ret$sd / sqrt(ceiling(input$N))
            }
            
            return(ret)
        })
        
        output$densPlot <- renderPlot({
            if( currentData()$discrete ){
                plot(currentData()$x, currentData()$y, main="Sannolikhetsfunktion",
                     xlab="", ylab="", type="h", col="blue",
                     xlim=currentData()$xlim)
            }else{
                plot(currentData()$x, currentData()$y, main="T\u00e4thetsfunktion",
                     xlab="", ylab="", type="l", col="blue",
                     xlim=currentData()$xlim)
            }
            points(currentData()$xSim.1, rep(0,ceiling(input$N)), pch=4, col="black")
            points(mean(currentData()$xSim.1), 0, pch=1, col="red")
        })
        output$histPlot <- renderPlot({
            if( input$norm ){
                x <- seq(currentData()$mu-3*currentData()$sd,
                         currentData()$mu+3*currentData()$sd, length.out=1e3)
            }else{
                x <- range(currentData()$xSim)
            }
            H <- hist(currentData()$xSim, ylab="", breaks=20,
                      col="lightblue", xlim=range(x),
                      xlab=sprintf("E = %.2g, D = %.2g",
                                   currentData()$mu, currentData()$sd), 
                      main=paste("Histogram \u00f6ver 1000 simulerade",
                                 ifelse(input$isSum,"summor","medelv\u00e4rden")) )
            
            if( input$norm ){
                lines(x, dnorm(x, currentData()$mu, currentData()$sd)*
                         (H$breaks[2]-H$breaks[1])*length(currentData()$xSim),
                      lwd=2, col="red")
            }
        })
        output$normPlot <- renderPlot({
            req( input$norm )
            qqnorm(currentData()$xSim, col="blue", xlab="", ylab="",
                   main="Normalf\u00f6rdelning")
            qqline(currentData()$xSim)
        })
    }

    ##the ui for cgsgui
    cgsgui.ui <- fluidPage(
        ##title
        titlePanel("CGSgui"),

        ##Grid layout
        fluidRow(
            ##First plot
            column(6, plotOutput("densPlot")),
            ##sidebar with various inputs
            column(6,
                   selectInput("dens", "", choices=choices),
                   conditionalPanel(
                       condition = "input.dens == 'exp'",
                       numericInput("mu_exp", "\u03bc", 1, min=0)
                   ),
                   conditionalPanel(
                       condition = "input.dens == 'unif'",
                       fluidRow(column(3, numericInput("a_unif", "a", 0)),
                                column(3, numericInput("b_unif", "b", 1)))
                   ),
                   conditionalPanel(
                       condition = "input.dens == 'norm'",
                       fluidRow(column(3, numericInput("mu_norm", "\u03bc", 0)),
                                column(3, numericInput("sigma_norm", "\u03c3",
                                                       1, min=0)))
                   ),
                   conditionalPanel(
                       condition = "input.dens == 'binom'",
                       fluidRow(column(3, numericInput("n_binom", "n", 10, min=1)),
                                column(3, numericInput("p_binom", "p", 0.5,
                                                       min=0, max=1)))
                   ),
                   conditionalPanel(
                       condition = "input.dens == 'pois'",
                       numericInput("mu_pois", "\u03bc", 1, min=0)
                   ),
                   fluidRow(column(3, numericInput("N", "N", 1, min=1)),
                            column(6, radioButtons("isSum", "", inline=TRUE,
                                                   choiceNames=c("Sum","Mean"),
                                                   choiceValues=c(TRUE,FALSE)))),
                   fluidRow(column(3,checkboxInput("norm", "Norm", FALSE)),
                            column(3,actionButton("plot", "Plot")))
                   )
        ),
        fluidRow(
            ##Second plot
            column(6, plotOutput("histPlot")),
            ##Third plot
            column(6, plotOutput("normPlot"))
        )
    )
    ##launch the GUI
    runApp(shinyApp(ui=cgsgui.ui, server=cgsgui.server), quiet=TRUE)
}

