#' Grafic user interface illustrating confidence intervalls
#'
#' Grafic user interface for illustrating confidence intervalls for mu in
#' N(mu,sigma) distributions
#' 
#' @return Nothing
#' @import shiny
#' @export
#' @examples
#' ## Only run these examples in interactive R sessions
#' if( interactive() ){
#'   konfgui()
#' }
konfgui <- function(){
    ##general information about exiting the app
    print("Use escape or ctrl+c to exit")

#######################################
    ## Internatl helper functions - GUIs ##
#######################################
    ##predefine text strings to fix UNI-code
    main.1 <- "X ~ N(\u03bc, \u03c3)"
    main.2 <- "\u03bc\u002a = X\u0304 ~ N(\u03bc, \u03c3/\u221an)"
    ##the server for konfgui
    konfgui.server <- function(input, output, session){
        ##First a reactive component generating plots
        currentData <- reactive({
            ##require that some parameters are OK
            req(input$n > 0, input$plot)
            ##compute densities
            ret <- list( x=seq(input$mu-3.5*input$sd,
                               input$mu+3.5*input$sd, length.out=1e3) )
            ret$y <- dnorm(ret$x, input$mu, input$sd)
            ##and for the average
            ret$sd.avg <- input$sd/sqrt(input$n)
            ret$y.avg <- dnorm(ret$x, input$mu, ret$sd.avg)
            ##range for the plots
            ret$xlim <- range(ret$x)
            if( input$scale ){
                ret$xlim.avg <- input$mu + 3.5*ret$sd.avg*c(-1,1)
            }else{
                ret$xlim.avg <- ret$xlim
            }
            ##sample data
            ret$xSim <- rnorm(input$n, input$mu, input$sd)
            ret$muSim <- mean(ret$xSim)
            
            return(ret)
        })
        output$densPlot <- renderPlot({
            plot(currentData()$x, currentData()$y, xlab="", ylab="",
                 type="l", col="blue", main=main.1,
                 xlim=currentData()$xlim)
            points(currentData()$xSim, rep(0,input$n), pch=4, col="black")
            points(currentData()$muSim, max(currentData()$y)*0.05, pch=1, col="red")
        })
        output$intPlot <- renderPlot({
            f.max <- max(currentData()$y.avg)
            plot(currentData()$x, currentData()$y.avg, xlab="", ylab="",
                 type="l", col="blue", main=main.2,
                 xlim=currentData()$xlim.avg, ylim=c(-0.1*f.max, f.max))
            lines(c(input$mu,input$mu), c(-0.1,1)*f.max, col="red")
            ##plot critical area
            k <- plotCritArea(input$mu, currentData()$sd.avg, input$alpha,
                              input$intervall, col='blue', border='blue')
            ##compute intervalls
            CI <- list()
            if( input$intervall == "2" ){
                CI$k.un <- (currentData()$muSim +
                            qnorm(input$alpha/2)*currentData()$sd.avg)
                CI$k.ov <- (currentData()$muSim -
                            qnorm(input$alpha/2)*currentData()$sd.avg)
            }else if( input$intervall == "<" ){
                CI$k.un <- currentData()$xlim.avg[1]-10
                CI$k.ov <- (currentData()$muSim -
                            qnorm(input$alpha)*currentData()$sd.avg)
            }else if( input$intervall == ">" ){
                CI$k.un <- (currentData()$muSim +
                            qnorm(input$alpha)*currentData()$sd.avg)
                CI$k.ov <- currentData()$xlim.avg[2]+10
            }
            CI$sign <- !(CI$k.un<input$mu && CI$k.ov>input$mu)
            ##reference width
            if( input$intervall == "2" ){
                lines(c(k$k.un,k$k.ov), c(0,0), col="green")
            }else if( input$intervall == "<" ){
                lines(c(k$k.un,0), c(0,0), col="green")
                lines(c(0,currentData()$xlim.avg[2]+10), c(0,0), col="black")
            }else if( input$intervall == ">" ){
                lines(c(0,k$k.ov), c(0,0), col="green")
                lines(c(currentData()$xlim.avg[1]-10,0), c(0,0), col="black")
            }
            ##plot intervallsfun
            if( input$intervall == "2" ){
                lines(c(CI$k.un,CI$k.ov), c(-0.05,-0.05)*f.max,
                      col=c("green","red")[(CI$sign)+1])
            }else if( input$intervall == "<" ){
                lines(c(currentData()$muSim,CI$k.ov), c(-0.05,-0.05)*f.max,
                      col=c("green","red")[(CI$sign)+1])
                lines(c(CI$k.un,currentData()$muSim), c(-0.05,-0.05)*f.max,
                      col=c("purple","red")[(CI$sign)+1])
            }else if( input$intervall == ">" ){
                lines(c(CI$k.un,currentData()$muSim), c(-0.05,-0.05)*f.max,
                      col=c("green","red")[(CI$sign)+1])
                lines(c(currentData()$muSim,CI$k.ov), c(-0.05,-0.05)*f.max,
                      col=c("purple","red")[(CI$sign)+1])
            }
            points(c(1,1)*currentData()$muSim, c(0,-0.05*f.max), pch=1, col="red")
        })
    }

    ##the ui for konfgui
    konfgui.ui <- fluidPage(
        ##title
        titlePanel("Konfgui"),

        ##panel layout [plots control]
        sidebarLayout(
            ##sidebar with various inputs
            sidebarPanel(
                fluidRow(
                    column(5, numericInput("mu", "\u03bc", value=0)),
                    column(5, numericInput("sd", "\u03c3", value=1, min=0))
                ),
                fluidRow(
                    column(5, numericInput("alpha", "\u03b1",
                                           value=0.05, min=0, max=1)),
                    column(5, numericInput("n", "n", value=1, min=1))
                ),
                checkboxInput("scale", "scale", FALSE),
                selectInput("intervall", "", 
                            choices=c("tv\u00e5sidigt" = "2",
                                      "upp bgr." = "<",
                                      "ned bgr." = ">") ),
                actionButton("plot", "Plot")
            ),
            ##main panel containing the plots
            mainPanel(
                plotOutput("densPlot"),
                plotOutput("intPlot")
            ),
            ##sidebar to the right
            position = "right")
    )

    ##launch the GUI
    runApp(shinyApp(ui=konfgui.ui, server=konfgui.server), quiet=TRUE)
}

