#' Grafic user interface illustrating linear and polynomial regression
#'
#' Grafic user interface for linear and polynomial regression. Plots the fitted
#' regression line, residuals and normplot for residuals. Prediction intervalls
#' and values can be obtained by clicking in the figure. Regression line is blue
#' if all parameters are significant, red otherwise. For simple linear
#' regression calibration regions are also given; these are based on graphical
#' inversion of the corresponding prediction intervalls.
#'
#' @param x Vector of explanatory variables. 
#' @param y Vector of dependent variables. 
#' @return Nothing
#' @import shiny stats graphics
#' @export
#' @examples
#' ## Only run these examples in interactive R sessions
#' if( interactive() ){
#'   x <- runif(10)
#'   y <- 1 + 0.5*x + 0.1*rnorm(10)
#'   reggui(x,y)
#' }
reggui <- function(x, y){
    if( length(x)!=length(y) || !is.vector(x) || !is.vector(y) ){
        stop("x and y must be vectors of same length")
    }
    values <- reactiveValues(x=x, y=y)

    ##general information about exiting the app
    print("Use escape or ctrl+c to exit")

#######################################
    ## Internatl helper functions - GUIs ##
#######################################
    ##the server for reggui
    reggui.server <- function(input, output, session){
        ##Define reactive value for plot clicking
        x.click <- reactiveValues(x=NULL)
        ##First a reactive component generating plots
        currentData <- reactive({
            ##require that some parameters are OK
            req(input$p>0, input$p<=10, round(input$p)==input$p,
                input$alpha>0, input$alpha<1)
            ##create return structure
            x <- values$x
            y <- values$y
            ##check that degree isn't to high
            req(input$p <= (length(x)-2))
            ##set up equation (should we have zero mean?)
            f <- sprintf("y ~ poly(x,degree=%.0f,raw=TRUE)%s",
                         input$p, c("","-1")[input$int0+1])
            f <- as.formula(f)
            ##do regression
            res <- lm(f, data.frame(x=x,y=y))
            ##also do predictions (for plotting)
            x.pred <- seq(min(x), max(x), length.out=1000)
            y.CI <- predict(res, data.frame(x=x.pred),
                            level=input$alpha, interval=c("confidence"))
            y.PI <- predict(res, data.frame(x=x.pred),
                            level=input$alpha, interval=c("prediction"))
            ##and predict at clicked point (assume NULL for no click)
            y.click.CI <- NULL
            y.click.PI <- NULL
            y.click.KI <- NULL
            if(is.numeric(x.click$x)){
                y.click.CI <- predict(res, data.frame(x=x.click$x),
                                      level=input$alpha, interval=c("confidence"))
                y.click.PI <- predict(res, data.frame(x=x.click$x),
                                      level=input$alpha, interval=c("prediction"))
                ##Calibration. Only valid for order 1 and beta significant
                if(input$p==1 &&  prod(confint(res)[res$rank,])>0){
                    y0 <- y.click.CI[1,"fit"]
                    if(input$int0){ ## y=b*x
                        x.bar <- 0
                        const <- 1
                        beta <- coef(res)[1]
                    }else{ ## y=a+b*x
                        x.bar <- mean(x)
                        y0 <- y0 - mean(y)
                        const <- 1+1/length(x)
                        beta <- coef(res)[2]
                    }
                    Sxx <- sum( (x-x.bar)^2 )
                    t.alpha <- qt(p=(1+input$alpha)/2, df=res$df.residual)
                    c <- beta^2 - (t.alpha*summary(res)$sigma)^2 / Sxx
                    d <- c*const + y0^2/Sxx
                    if( d>0 ){
                        d <- summary(res)$sigma*t.alpha/c * sqrt(d)
                        y.click.KI <- x.bar + beta*y0/c + c(-d,d)
                    }
                }
            }
            ##check if all parameters are significant
            ##we check the signs, non-sgnificant -> different signs -> rowSums=0
            suppressWarnings( all.sign <- all(rowSums(sign(confint(res)))!=0) )
            ##return values
            return(list(x=x, y=y, res=res, x.pred=x.pred, all.sign=all.sign,
                        y.pred=cbind(y.PI[,"lwr"], y.CI[,"lwr"], y.CI[,"fit"],
                                     y.CI[,"upr"], y.PI[,"upr"]),
                        y.click=cbind(y.click.PI[,"lwr"], y.click.CI[,"lwr"],
                                      y.click.CI[,"fit"], y.click.CI[,"upr"],
                                      y.click.PI[,"upr"]),
                        x.cal=y.click.KI))
        })
        ##observe plot clicking
        observeEvent(input$click_xy,{x.click$x <- input$click_xy$x})
        ##plots
        output$lmPlot <- renderPlot({
            ##extract x-values
            x <- currentData()$x
            x.pred <- currentData()$x.pred
            ##plots
            plot(x, currentData()$y, pch=19, col="blue",
                 xlim=range(x), ylim=range(currentData()$y.pred),
                 xlab="x", ylab="y", main="Linear Regression")
            if(input$grid){
                grid(NULL, NULL, lty=6, col="lightgrey")
            }
            lines(x.pred, currentData()$y.pred[,3], lty=1,
                  col=c("red","blue")[currentData()$all.sign+1])
            if(input$CI){
                lines(x.pred, currentData()$y.pred[,2], lty=4, col="blue")
                lines(x.pred, currentData()$y.pred[,4], lty=4, col="blue")
            }
            if(input$PI){
                lines(x.pred, currentData()$y.pred[,1], lty=2, col="red")
                lines(x.pred, currentData()$y.pred[,5], lty=2, col="red")
            }
            if(input$mark && is.numeric(x.click$x)){
                x <- c(par("usr")[1], x.click$x)
                y <- currentData()$y.click[3]
                lines(x, currentData()$y.click[3]*c(1,1), lty=1,
                      col=c("red","blue")[currentData()$all.sign+1])
                if(input$KI && !is.null(currentData()$x.cal)){
                    lines(c(x.click$x, currentData()$x.cal[2]),
                          currentData()$y.click[3]*c(1,1), lty=1,
                          col=c("red","blue")[currentData()$all.sign+1])
                    lines(c(currentData()$x.cal[1], currentData()$x.cal[1]),
                          c(par("usr")[3],y), lty=1, col="darkgreen")
                    lines(c(currentData()$x.cal[2], currentData()$x.cal[2]),
                          c(par("usr")[3],y), lty=1, col="darkgreen")
                }
                if(input$CI){
                    lines(x, currentData()$y.click[2]*c(1,1), lty=4, col="blue")
                    lines(x, currentData()$y.click[4]*c(1,1), lty=4, col="blue")
                    y <- currentData()$y.click[4]
                }
                if(input$PI){
                    lines(x, currentData()$y.click[1]*c(1,1), lty=2, col="red")
                    lines(x, currentData()$y.click[5]*c(1,1), lty=2, col="red")
                    y <- currentData()$y.click[5]
                }
                lines(x.click$x*c(1,1), c(par("usr")[3],y), lty=1, col="blue")
            }
        })
        output$resPlot <- renderPlot({
            x <- switch(input$resX,
                        x = currentData()$x,
                        nr = 1:length(currentData()$x),
                        y = currentData()$y)
            xlab <- switch(input$resX, x="x", nr="nr", y="y")
            plot(x=x, y=currentData()$res$residuals, pch=19,
                 col="blue", ylab="", xlab=xlab, main="Residuals")
            if(input$grid){
                grid(NULL, NULL, lty=6, col="lightgrey")
            }else{
                abline(h=0, lty=6, col="lightgrey")
            }
        })
        output$normPlot <- renderPlot({
            qqnorm(currentData()$res$residuals, col="blue", pch=3,
                   xlab="", ylab="", main="Normplot of Residuals")
            qqline(currentData()$res$residuals, lty=2, col="darkgrey")
            if(input$grid){
                grid(NULL, NULL, lty=6, col="lightgrey")
            }
        })
        output$fitsText <- renderUI({
            suppressWarnings( summary.res <- summary(currentData()$res) )
            text.Q0 <- sprintf("Q0 = %.4f", sum(currentData()$res$residuals^2))
            text.s <- sprintf("s = %.4f", summary.res$sigma)
            text.R2 <- sprintf("R2 = %.4f", summary.res$r.squared)
            b <- coefficients(currentData()$res)
            suppressWarnings( CI.b <- confint(currentData()$res) )
            text.beta <- ""
            for(i in 1:length(b)){
                text.beta <- paste(text.beta,
                                   sprintf("%.3g (%.3g, %.3g)",
                                           b[i], CI.b[i,1], CI.b[i,2]),
                                   sep="<br/>")
            }
            HTML(paste(text.Q0, text.s, text.R2, "b* =",
                       substr(text.beta, 6, nchar(text.beta)),
                       sep="<br/>"))
        })
        output$clickText <- renderUI({
            req( x.click$x )
            if( is.null(currentData()$x.cal) ){
                T <- HTML("<center>",
                          sprintf("x: %.4g, y: %.4g, Conf: (%.4g, %.4g), Pred: (%.4g, %.4g)",
                                  x.click$x, currentData()$y.click[3],
                                  currentData()$y.click[2], currentData()$y.click[4],
                                  currentData()$y.click[1], currentData()$y.click[5]),
                          "<center/>")
            }else{
                T <- HTML("<center>",
                          sprintf("x: %.4g, y: %.4g, Conf: (%.4g, %.4g), Pred: (%.4g, %.4g), Cal: (%.4g, %.4g)",
                                  x.click$x, currentData()$y.click[3],
                                  currentData()$y.click[2], currentData()$y.click[4],
                                  currentData()$y.click[1], currentData()$y.click[5],
                                  currentData()$x.cal[1], currentData()$x.cal[2]),
                          "<center/>")
            return(T)
        }
    
        })
        ##constant text fields since I cant find a simpler way of doing it
        output$estText <- renderText("Estimated parameters")
        output$modelText <- renderText("Model")
        output$viewText <- renderText("View")
        output$actionText <- renderText("Action")
    }
    
    ##the ui for reggui
    reggui.ui <- fluidPage(
        ##title
        titlePanel("RegGui"),
        
        ##Grid layout
        fluidRow(
            ##left hand columns with plots
            column(8, htmlOutput("clickText"),
                   plotOutput("lmPlot", click="click_xy"),
                   fluidRow(column(6,plotOutput("resPlot")),
                            column(6,plotOutput("normPlot")))
                   ),
            ##sidebar with various inputs
            column(4, h3(textOutput("estText")), htmlOutput("fitsText"),
                   fluidRow(column(6,
                                   h3(textOutput("modelText")),
                                   numericInput("p", "Degree", 1, min=1, max=10),
                                   numericInput("alpha", "Confidence", 0.95, min=0, max=1),
                                   checkboxInput("int0", "Intercept = 0", FALSE)
                                   ),
                            column(6,
                                   h3(textOutput("viewText")),
                                   checkboxInput("CI", "Conf. Int.", FALSE),
                                   checkboxInput("PI", "Pred. Int.", FALSE),
                                   checkboxInput("KI", "Cal. Int.", FALSE),
                                   checkboxInput("grid", "Grid", TRUE),
                                   selectInput("resX", "Residuals as fcn. of",
                                               choices=c("x" = "x",
                                                         "nr" = "nr",
                                                         "y" = "y"))
                                   )
                            ),
                   h3(textOutput("actionText")),
                   checkboxInput("mark", "Mark Int.", FALSE)
                   )
        )
    )
    ##launch the GUI
    runApp(shinyApp(ui=reggui.ui, server=reggui.server), quiet=TRUE)
}
