#' Grafic interface illustrating the effect of correlated residuals on regression
#'
#' Grafic user interface for illustrating how correlated residuals, following an
#' AR(1)-process, affects confidence intervalls and coverage when estimating
#' beta1 and beta2 in linear regression under assumptions of i.i.d. residuals.
#'
#' The function illustrates how the estimates behave if data from
#' \deqn{y_i = \beta_0 + \beta_1 \cdot x_i + e_i,\qquad e_i \sim AR(1)}{y_i = beta0 + beta1 * x_i + e_i,    e_i ~ AR(1)}
#' is analysed using the "standard" model
#' \deqn{y_i = \beta_0 + \beta_1 \cdot x_i + e_i,\qquad e_i \sim N(0,1)\text{ and i.i.d.}}{y_i = beta0 + beta1 * x_i + e_i,   e_i ~ N(0,1) and i.i.d.}
#' 
#' @return Nothing
#' @import shiny
#' @export
#' @examples
#' ## Only run these examples in interactive R sessions
#' if( interactive() ){
#'   tidsseriegui()
#' }
tidsseriegui <- function(){
    ##general information about exiting the app
    print("Use escape or ctrl+c to exit")
    ##launch the GUI
    runApp(shinyApp(ui=tidsserieGUI.ui, server=tidsserieGUI.server), quiet=TRUE)
}

#######################################
## Internatl helper functions - GUIs ##
#######################################
##the server for tidsserieGUI
tidsserieGUI.server <- function(input, output, session){
    ##First a reactive component generating plots
    currentData <- reactive({
        ##require that some parameters are OK
        req(is.numeric(input$beta0), is.numeric(input$beta1),
            -1<input$alpha, input$alpha<1, input$sigma>0, input$plot)
        ##simulate 1000 repetitions of N=100 samples from a regression model
        ##with correlated errors simulate errors
        if( input$alpha!=0 ){
            e <- as.vector(arima.sim(list(ar=input$alpha), sd=input$sigma,
                                     n=100*1000))
        }else{
            e <- rnorm(n=100*1000, sd=input$sigma)
        }
        e <- matrix(e, 100, 1000)
        #and then observations
        x <- 1:dim(e)[1]
        y <- input$beta0 + input$beta1*x + e
        ##compute squared sums
        Sxx <- sum( (x-mean(x))^2 )
        Syy <- apply(y, 2, function(z){sum((z-mean(z))^2)}) 
        Sxy <- apply(y, 2, function(z){sum((x-mean(x))*(z-mean(z)))})
        ##compute beta estimates
        beta1.est <- Sxy/Sxx
        beta0.est <- colMeans(y)-beta1.est*mean(x)
        ##compute variance estimate
        sigma.est <- sqrt(colSums( (y- cbind(1,x) %*%
                                    rbind(beta0.est,beta1.est))^2 ) /
                          (length(x)-2))
        ##compute confidence intervalls
        beta0.sd <- sqrt(1/length(x)+mean(x)^2 /Sxx)
        beta1.sd <- sqrt(1/Sxx)
        t.q <- qt(p=0.975,df=length(x)-2)
        beta0.CI <- rbind(beta0.est-t.q*beta0.sd*sigma.est,
                          beta0.est+t.q*beta0.sd*sigma.est)
        beta1.CI <- rbind(beta1.est-t.q*beta1.sd*sigma.est,
                          beta1.est+t.q*beta1.sd*sigma.est)
        ##collect outputs
        ret <- list(x=x, y=y[,1],
                    ##regression for the first series
                    beta=c(beta0.est[1],beta1.est[1]),
                    sigma=sigma.est[1],
                    res=y[,1]-beta0.est[1]-beta1.est[1]*x,
                    ##CIs for all regressions
                    beta0.CI=t(beta0.CI), beta1.CI=t(beta1.CI))
        return(ret)
    })
    output$lmPlot <- renderPlot({
        plot(currentData()$x, currentData()$y, xlab="", ylab="",
             type=c("p","l")[(input$line)+1], col="blue",
             main="Realisering och anpassad regressionslinje")
        abline(currentData()$beta, col="red")
    })
    output$resPlot <- renderPlot({
        plot(currentData()$res, type="l", xlab="", ylab="",
             main="Residualer", col="blue")
        abline(h=0, col="grey")
    })
    output$normPlot <- renderPlot({
        qqnorm(currentData()$res, col="blue", xlab="", ylab="",
               main="Normalf\u00f6rdelning f\u00f6r residualer")
        qqline(currentData()$res)
    })
    output$acfPlot <- renderPlot({
        plot(acf(currentData()$res), ylab="", xlab="Lag",
             main="Autokorrelation f\u00f6r residualer")
    })
    output$fitsText <- renderUI({
        req(is.numeric(input$beta0), is.numeric(input$beta1))
        if( input$beta0!=0 ){
            ##beta0 is significant if both edges of CI have same sign
            is.sign <- (sign(currentData()$beta0.CI[1,1]) ==
                        sign(currentData()$beta0.CI[1,2]))
            text.beta0 <- sprintf("&beta;<sub>0</sub>%s = %.3f (%.3f, %.3f)",
                                  c("","*")[is.sign+1],
                                  currentData()$beta[1],
                                  currentData()$beta0.CI[1,1],
                                  currentData()$beta0.CI[1,2])
        }else{
            text.beta0 <- "&beta;<sub>0</sub> = 0.000"
        }
        if( input$beta1!=0 ){
            ##beta1 is significant if both edges of CI have same sign
            is.sign <- (sign(currentData()$beta1.CI[1,1]) ==
                        sign(currentData()$beta1.CI[1,2]))
            text.beta1 <- sprintf("&beta;<sub>1</sub>%s = %.3f (%.3f, %.3f)",
                                  c("","*")[is.sign+1],
                                  currentData()$beta[2],
                                  currentData()$beta1.CI[1,1],
                                  currentData()$beta1.CI[1,2])
        }else{
            text.beta1 <- "&beta;<sub>1</sub> = 0.000"
        }
        HTML(paste("Skattade parametrar (95 procentigt intervall)",
                   text.beta0, text.beta1,
                   sprintf("&sigma; = %.3f", currentData()$sigma),
                   sep="<br/>"))
    })
    output$beta0Plot <- renderPlot({
        ##require that some parameters are OK
        req(-1<input$alpha, input$alpha<1, input$sigma>0, input$beta0!=0)
        ##precomputed values
        ret <- currentData()
        ##check CI for significance
        I <- ret$beta0.CI[,1]<input$beta0 & input$beta0<ret$beta0.CI[,2]
        ##compute intervall widths for plotting
        width <- 1.2*max( abs(ret$beta0.CI[1:100,]-input$beta0) )
        ##use only the first 100 for plotting
        plot(0, 0, xlim=c(input$beta0-width, input$beta0+width), ylim=c(0,101),
             type='n', ylab='', main="Konfidensintervall f\u00f6r \u03b20",
             xlab=sprintf('Andel av 1000 intervall som missar \u03b20: %1.3f',
                          1-mean(I)))
        for(i in 1:100){
            lines(ret$beta0.CI[i,], c(i,i), col=c('red', 'blue')[I[i]+1])
        }
        abline(v=input$beta0, col='red')
    })
    output$beta1Plot <- renderPlot({
        ##require that some parameters are OK
        req(-1<input$alpha, input$alpha<1, input$sigma>0, input$beta1!=0)
        ##precomputed values
        ret <- currentData()
        ##check CI for significance
        I <- ret$beta1.CI[,1]<input$beta1 & input$beta1<ret$beta1.CI[,2]
        ##compute intervall widths for plotting
        width <- 1.2*max( abs(ret$beta1.CI[1:100,]-input$beta1) )
        ##use only the first 100 for plotting
        plot(0, 0, xlim=c(input$beta1-width, input$beta1+width), ylim=c(0,101),
             type='n', ylab='', main="Konfidensintervall f\u00f6r \u03b21",
             xlab=sprintf('Andel av 1000 intervall som missar \u03b21: %1.3f',
                          1-mean(I)))
        for(i in 1:100){
            lines(ret$beta1.CI[i,], c(i,i), col=c('red', 'blue')[I[i]+1])
        }
        abline(v=input$beta1, col='red')
    })
}

##the ui for tidsserieGUI
tidsserieGUI.ui <- fluidPage(
    ##title
    titlePanel("Tidsserie GUI"),

    ##Grid layout
    fluidRow(
        ##First plot
        column(8, plotOutput("lmPlot")),
        ##sidebar with status text and inputs
        column(4,
               htmlOutput("fitsText"),
               fluidRow(column(4, checkboxInput("line", "Draw line", FALSE),
                               offset=4)),
               fluidRow(column(4, numericInput("beta0", "\u03b20", value=0)),
                        column(4, numericInput("beta1", "\u03b21", value=0))),
               fluidRow(column(4, numericInput("alpha", "\u03b1", value=0)),
                        column(4, numericInput("sigma", "\u03c3",
                                               value=1, min=0)) 
                        ),
               actionButton("plot", "Plot")
               )
    ),
    fluidRow(
        ##Second plot
        column(4, plotOutput("resPlot")),
        ##Third plot
        column(4, plotOutput("normPlot")),
        ##Fourth plot
        column(4, plotOutput("acfPlot"))
    ),
    fluidRow(
        ##Second plot
        column(6, plotOutput("beta0Plot")),
        ##Third plot
        column(6, plotOutput("beta1Plot"))
    )
)
